﻿using ByteSprite.DependencyInjection;
using UnityEngine;

namespace Plants {
	public class Cursor : MonoBehaviour {
		public enum CursorState {
			None,
			NewPlant,
			MovePlant,
			Water,
			RemovePlant
		}

		[SerializeField]
		GameObject sparkle, dust;

		[Inject]
		InputManager inputManager;

		[Inject]
		PlantManager plantManager;

		[SerializeField]
		PlantGlow plantGlow;

		[SerializeField]
		PlantGlow showPlantGlow;

		[SerializeField]
		Status light, water;

		[SerializeField]
		Animator waterCan;

		float waterTime;

		bool IsWatering { get { return waterTime > 0f; } }

		CursorState state = CursorState.None;

		ITile target1, target2;

		public CursorState State {
			get {
				return state;
			}
			set {
				state = value;

				switch(state){
					case CursorState.Water:
						WaterState();
					break;
					case CursorState.MovePlant:
						MoveState ();
					break;
					case CursorState.RemovePlant:
						RemoveState();
					break;
				}
			}
		}

		new SpriteRenderer renderer;

		Plant selectedPlant;
		public Plant SelectedPlant {
			get { return selectedPlant; }
			set {
				selectedPlant = value;
				renderer.sprite = SelectedPlant?.GetComponent<SpriteRenderer> ().sprite ?? SelectedPlant?.Juvi;
				showPlantGlow.GetComponent<SpriteRenderer> ().sprite = SelectedPlant?.GetComponent<SpriteRenderer>().sprite ?? SelectedPlant?.Juvi;
			}
		}

		bool canPlace = true;
		public bool CanPlace { get { return canPlace; }
			set {
				canPlace = value;
				renderer.color = canPlace ? Color.green : Color.red; 
				if(State == CursorState.MovePlant && canPlace)
					renderer.color = Color.gray;
			} 
		}

		void Awake () {
			Injector.Inject(this);

			inputManager.OnSelect += OnSelect;

			renderer = GetComponent<SpriteRenderer>();
		}

		void OnDestroy () {
			inputManager.OnSelect -= OnSelect;
		}

		void Update () {
			waterCan.gameObject.SetActive(State == CursorState.Water);

			ShowStats();

			waterTime -= Time.deltaTime;

			if(IsWatering){
				return;
			}

			var tiles = FindObjectsOfType<Tile>();

			foreach(var tile in tiles){
				tile.GetComponent<SpriteRenderer>().color = SelectedPlant != null ? new Color(1f,1f,1f,0.1f) : Color.clear;
			}


			if(SelectedPlant != null)
				plantManager.HoverPlant (transform.position, null);
			else if(inputManager.Hovering?.Occupant?.GetType()==typeof(Plant))
				plantManager.HoverPlant(transform.position, (Plant)inputManager.Hovering.Occupant);
			else
				plantManager.HoverPlant(null, null);

			transform.position = (Vector2)Camera.main.ScreenToWorldPoint(Input.mousePosition);

			CanPlace = false;
			if(inputManager.Hovering != null){
				if(inputManager.Hovering.Occupant == null || State == CursorState.MovePlant)
					if(SelectedPlant != null)
						if(SelectedPlant.Location == inputManager.Hovering.Location)
							CanPlace = true;
			}


			if (state != CursorState.NewPlant)
				return;

			if (Input.GetMouseButtonDown (0)) {
				if (!CanPlace) {
					SelectedPlant = null;
					State = CursorState.None;
				}
				else {
					PlacePlant ();
				}
			}
		}

		void ShowStats(){
			water.transform.parent.gameObject.SetActive (false);
			light.transform.parent.gameObject.SetActive (false);
			if (inputManager.Hovering?.Occupant != null && SelectedPlant == null) {
				water.transform.parent.gameObject.SetActive(true);
				light.transform.parent.gameObject.SetActive (true);

				var tile = inputManager.Hovering;
				var plant = (Plant)tile?.Occupant;

				water.IsSatisfied = plant.CurrentWater > 0.1f && plant.CurrentWater < 0.9f;
				light.IsSatisfied = tile.Lighting == plant.RequiredLight;
			}
		}

		void PlacePlant() {
			State = CursorState.None;

			if (SelectedPlant == null)
				return;

			var plant = Instantiate (SelectedPlant, inputManager.Selected.Position, Quaternion.identity);
			Destroy(Instantiate(sparkle, inputManager.Selected.Position, Quaternion.identity), 2f);
			inputManager.Selected.Occupant = plant;
			SelectedPlant = null;
			var glow = plant.GetComponent<Plant> ().PlantGlow = Instantiate (plantGlow, plant.transform);
			plant.PlantGlow = glow;
		}

		void WaterState() {
			//
		}

		void MoveState() {
			inputManager.Selected = null;
			target1 = null;
			target2 = null;
		}

		void OnSelect(ITile selected) {
			if(State == CursorState.RemovePlant || State == CursorState.None){
				var tile = inputManager.Hovering;
				var occupant = tile?.Occupant;
				if (occupant != null){
					if (!((Plant)occupant).IsAlive) {
						Instantiate(dust, tile.Position, Quaternion.identity);
						Destroy(((Plant)occupant).gameObject);
						tile.Occupant = null;
						return;
					}
				}
			}
			
			if(State == CursorState.MovePlant || State == CursorState.None){
				if (target1 == null){
					target1 = selected;
					SelectedPlant = (Plant)target1?.Occupant;
				}
				else {
					if(canPlace) {
						target2 = selected;
						plantManager.MoveOccupant (target1, target2);
					}
					SelectedPlant = null;
					target1 = null;
					target2 = null;
					State = CursorState.None;
				}
			}

			if (State == CursorState.Water){
				var occupant = inputManager.Selected?.Occupant;
				if(occupant != null){
					((Plant)occupant).Water();
					waterTime = 1f;
					waterCan.SetTrigger("Water");
				} else {
					State = CursorState.None;
				}
			}
		}

		void RemoveState(){
			//
		}
	}
}